"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const { __ΩClassType } = require('@deepkit/core');
const { __ΩIGridRequestDto } = require('./GridRequestDto');
const { __ΩFilterQuery } = require('@deepkit/orm');
const { __ΩIGridRequestDtoFilter } = require('./GridRequestDto');
function __assignType(fn, args) {
    fn.__type = args;
    return fn;
}
const __ΩRecord = ['K', 'T', 'l\'e#"Rb!b"Pde"!N#!'];
const orm_1 = require("@deepkit/orm");
const GridRequestDto_1 = require("./GridRequestDto");
const GridResponse_1 = __importDefault(require("./GridResponse"));
class AGrid {
    db;
    filterableColumns = null;
    sortableColumns = null;
    searchableColumns = null;
    constructor(db) {
        this.db = db;
    }
    async filter(dto) {
        let query = this.searchQuery(this.db.query(this.entity));
        const filters = this.addFilter(dto);
        if (!AGrid.isFilterEmpty(filters)) {
            query = query.filter(filters);
        }
        const count = await query.count();
        query = this.addSorter(query, dto);
        query = this.addPaging(query, dto);
        const result = await query.find();
        return new GridResponse_1.default(result, count, dto);
    }
    searchQuery(query) {
        return query;
    }
    static isFilterEmpty(filters) {
        if (!filters) {
            return true;
        }
        if (Object.keys(filters).length === 1 && filters.$and) {
            return filters.$and.length <= 0;
        }
        return false;
    }
    static createExpression(filter) {
        switch (filter.operator) {
            case GridRequestDto_1.Operator.EQ:
                return filter.value[0];
            case GridRequestDto_1.Operator.NEQ:
                return { $ne: filter.value[0] };
            case GridRequestDto_1.Operator.IN:
                return { $in: filter.value };
            case GridRequestDto_1.Operator.NIN:
                return { $nin: filter.value };
            case GridRequestDto_1.Operator.GT:
                return { $gt: filter.value[0] };
            case GridRequestDto_1.Operator.LT:
                return { $lt: filter.value[0] };
            case GridRequestDto_1.Operator.GTE:
                return { $gte: filter.value[0] };
            case GridRequestDto_1.Operator.LTE:
                return { $lte: filter.value[0] };
            case GridRequestDto_1.Operator.LIKE:
                return { $regex: filter.value[0] };
            case GridRequestDto_1.Operator.STARTS:
                return { $regex: `^${filter.value[0]}` };
            case GridRequestDto_1.Operator.ENDS:
                return { $regex: `${filter.value[0]}$` };
            case GridRequestDto_1.Operator.NEMPTY:
                return { $ne: null };
            case GridRequestDto_1.Operator.EMPTY:
                return { $eq: null };
            case GridRequestDto_1.Operator.BETWEEN:
                return {
                    $gte: filter.value[0],
                    $lte: filter.value[1],
                };
            case GridRequestDto_1.Operator.NBETWEEN:
                return {
                    $gte: filter.value[1],
                    $lte: filter.value[0],
                };
            default:
                return filter.value;
        }
    }
    addPaging(query, dto) {
        const paging = dto.paging ?? {
            page: 1,
            itemsPerPage: 10,
        };
        paging.page = Math.max(paging.page, 1);
        paging.itemsPerPage = Math.max(paging.itemsPerPage, 1);
        dto.paging = paging;
        return query
            .skip((paging.page - 1) * paging.itemsPerPage)
            .limit(paging.itemsPerPage);
    }
    addSorter(query, dto) {
        const allSorts = JSON.parse(JSON.stringify(dto.sorter ?? []));
        if (dto.additionalSorter) {
            allSorts.push(...dto.additionalSorter);
        }
        if (allSorts.length <= 0) {
            return query;
        }
        let sorted = query;
        allSorts.forEach(__assignType((sorter) => {
            let { column } = sorter;
            if (this.sortableColumns !== null) {
                if (column in this.sortableColumns) {
                    column = this.sortableColumns[column];
                }
                else {
                    throw new Error(`[${column}] is not allowed for sorting`);
                }
            }
            sorted = sorted.orderBy(column, sorter.direction.toLowerCase());
        }, ['sorter', '', 'P"2!"/"']));
        return sorted;
    }
    addFilter(dto) {
        const filter = { $and: [] };
        const allFilters = JSON.parse(JSON.stringify(dto.filter ?? []));
        if (dto.additionalFilter) {
            allFilters.push(...dto.additionalFilter);
        }
        if (allFilters.length > 0) {
            allFilters.forEach(__assignType((and) => {
                const orFilter = [];
                and.forEach(__assignType((or) => {
                    const column = this.getFilterableColumn(or.column);
                    orFilter.push({ [column]: AGrid.createExpression(or) });
                }, ['or', '', 'P"2!"/"']));
                if (filter.$and) {
                    filter.$and.push({ $or: orFilter });
                }
            }, ['and', '', 'P"2!"/"']));
        }
        if (dto.search) {
            const searches = [];
            this.searchableColumns?.forEach(__assignType((column) => {
                const name = this.getFilterableColumn(column);
                searches.push({ [name]: { $regex: dto.search } });
            }, ['column', '', 'P"2!"/"']));
            if (filter.$and) {
                filter.$and.push({ $or: searches });
            }
        }
        return filter;
    }
    getFilterableColumn(name) {
        let column = name;
        if (this.filterableColumns !== null) {
            if (column in this.filterableColumns) {
                column = this.filterableColumns[column];
            }
            else {
                throw new Error(`[${column}] is not allowed for filtering`);
            }
        }
        return column;
    }
    static __type = ['T', () => __ΩClassType, 'entity', () => __ΩRecord, 'filterableColumns', function () { return null; }, () => __ΩRecord, 'sortableColumns', function () { return null; }, 'searchableColumns', function () { return null; }, () => orm_1.Database, 'db', 'constructor', () => __ΩIGridRequestDto, 'dto', () => GridResponse_1.default, 'filter', () => orm_1.Query, 'query', () => orm_1.Query, 'searchQuery', () => __ΩFilterQuery, 'filters', 'isFilterEmpty', () => __ΩIGridRequestDtoFilter, 'createExpression', () => orm_1.Query, () => __ΩIGridRequestDto, () => orm_1.Query, 'addPaging', () => orm_1.Query, () => __ΩIGridRequestDto, () => orm_1.Query, 'addSorter', () => __ΩIGridRequestDto, () => __ΩFilterQuery, 'addFilter', 'name', 'getFilterableColumn', 'b!n"3#<=P&&o$#,J3%<>&P&&o\'#,J3(<>)P&F,J3*<>+PP7,2-;9"0.Pn/20Pe#!71`02PPe#!7324Pe#!7506<P"o7"28)09;sPn:22"0;;sPPe#!7<24n=20Pe#!7>0?;PPe#!7@24nA20Pe#!7B0C;PnD20e"!oE"0F;P&2G&0H;5'];
}
exports.default = AGrid;
