"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.__ΩICallbackSorter = exports.__ΩICallbackFilter = void 0;
const { __ΩIGridRequestDto } = require('./GridRequestDto');
const { __ΩIGridRequestDtoFilter } = require('./GridRequestDto');
const { __ΩOperator } = require('./GridRequestDto');
const { __ΩDirection } = require('./GridRequestDto');
function __assignType(fn, args) {
    fn.__type = args;
    return fn;
}
const __ΩRecord = ['K', 'T', 'l\'e#"Rb!b"Pde"!N#!'];
const GridRequestDto_1 = require("./GridRequestDto");
const GridResponse_1 = __importDefault(require("./GridResponse"));
const __ΩICallbackFilter = ['queryBuilder', 'column', () => __ΩOperator, 'operator', 'values', '', 'P!2!&2"n#2$&F2%!/&'];
exports.__ΩICallbackFilter = __ΩICallbackFilter;
const __ΩICallbackSorter = ['queryBuilder', 'column', () => __ΩDirection, 'direction', '', 'P!2!&2"n#2$!/%'];
exports.__ΩICallbackSorter = __ΩICallbackSorter;
class AKnexGrid {
    knex;
    filterableColumns = null;
    sortableColumns = null;
    searchableColumns = null;
    constructor(knex) {
        this.knex = knex;
    }
    async filter(dto) {
        let query = this.searchQuery(this.knex, dto);
        query = this.addFilters(query, dto);
        const countRes = await this.countQuery(query.clone().clear('select').clear('group'));
        query = this.addSorters(query, dto);
        query = this.addPaging(query, dto);
        const result = await query;
        const count = countRes[0]?.count ?? 0;
        return new GridResponse_1.default(result.map(__assignType((item) => this.toObject(item), ['item', '', 'P"2!"/"'])), count, dto);
    }
    async countQuery(knex) {
        return knex.count('*', { as: 'count' });
    }
    static addOrExpression(query, filter) {
        switch (filter.operator) {
            case GridRequestDto_1.Operator.EQ:
                return query.orWhere(filter.column, filter.value[0]);
            case GridRequestDto_1.Operator.NEQ:
                return query.orWhereNot(filter.column, filter.value[0]);
            case GridRequestDto_1.Operator.IN:
                return query.orWhereIn(filter.column, filter.value);
            case GridRequestDto_1.Operator.NIN:
                return query.orWhereNotIn(filter.column, filter.value);
            case GridRequestDto_1.Operator.GT:
                return query.orWhere(filter.column, '>', filter.value[0]);
            case GridRequestDto_1.Operator.LT:
                return query.orWhere(filter.column, '<', filter.value[0]);
            case GridRequestDto_1.Operator.GTE:
                return query.orWhere(filter.column, '>=', filter.value[0]);
            case GridRequestDto_1.Operator.LTE:
                return query.orWhere(filter.column, '<=', filter.value[0]);
            case GridRequestDto_1.Operator.LIKE:
                return query.orWhereLike(filter.column, `%${filter.value[0]}%`);
            case GridRequestDto_1.Operator.STARTS:
                return query.orWhereLike(filter.column, `${filter.value[0]}%`);
            case GridRequestDto_1.Operator.ENDS:
                return query.orWhereLike(filter.column, `%${filter.value[0]}`);
            case GridRequestDto_1.Operator.NEMPTY:
                return query.orWhereNotNull(filter.column);
            case GridRequestDto_1.Operator.EMPTY:
                return query.orWhereNull(filter.column);
            case GridRequestDto_1.Operator.BETWEEN:
                return query.orWhereBetween(filter.column, filter.value);
            case GridRequestDto_1.Operator.NBETWEEN:
                return query.orWhereNotBetween(filter.column, filter.value);
            default:
                return query.orWhere(filter.column, filter.value[0]);
        }
    }
    addPaging(query, dto) {
        const paging = dto.paging ?? {
            page: 1,
            itemsPerPage: 10,
        };
        paging.page = Math.max(paging.page, 1);
        paging.itemsPerPage = Math.max(paging.itemsPerPage, 1);
        dto.paging = paging;
        return query
            .offset((paging.page - 1) * paging.itemsPerPage)
            .limit(paging.itemsPerPage);
    }
    addSorters(query, dto) {
        let q = query;
        const allSorts = JSON.parse(JSON.stringify(dto.sorter ?? []));
        if (dto.additionalSorter) {
            allSorts.push(...dto.additionalSorter);
        }
        if (allSorts.length <= 0) {
            return q;
        }
        allSorts.forEach(__assignType((sorter) => {
            const sColumn = sorter.column;
            if (this.sortableColumns !== null && sColumn in this.sortableColumns) {
                const column = this.sortableColumns[sColumn];
                if (typeof column === 'string') {
                    q = q.orderByRaw(`${column} ${sorter.direction.toLowerCase()}`);
                }
                else {
                    q = column(q, sColumn, sorter.direction);
                }
            }
            else {
                throw new Error(`[${sColumn}] is not allowed for sorting`);
            }
        }, ['sorter', '', 'P"2!"/"']));
        return q;
    }
    addFilters(query, dto) {
        let q = query;
        const allFilters = JSON.parse(JSON.stringify(dto.filter ?? []));
        if (dto.additionalFilter) {
            allFilters.push(...dto.additionalFilter);
        }
        const that = this;
        if (allFilters.length > 0) {
            allFilters.forEach(__assignType((and) => {
                q = q.andWhere(function () {
                    let innerQ = this;
                    and.forEach(__assignType((or) => {
                        const column = that.getFilterableColumn(or.column);
                        if (typeof column === 'string') {
                            innerQ = AKnexGrid.addOrExpression(innerQ, {
                                column,
                                value: or.value,
                                operator: or.operator,
                            });
                        }
                        else {
                            innerQ = column(innerQ, or.column, or.operator, or.value);
                        }
                    }, ['or', '', 'P"2!"/"']));
                });
            }, ['and', '', 'P"2!"/"']));
        }
        if (dto.search && (this.searchableColumns?.length ?? 0) > 0) {
            q = q.andWhere(function () {
                let innerQ = this;
                that.searchableColumns?.forEach(__assignType((sColumn) => {
                    const column = that.getFilterableColumn(sColumn);
                    if (typeof column === 'string') {
                        innerQ = AKnexGrid.addOrExpression(innerQ, { column, value: [dto.search ?? ''], operator: GridRequestDto_1.Operator.LIKE });
                    }
                    else {
                        innerQ = column(innerQ, sColumn, GridRequestDto_1.Operator.LIKE, [dto.search ?? '']);
                    }
                }, ['sColumn', '', 'P"2!"/"']));
            });
        }
        return q;
    }
    getFilterableColumn(name) {
        if (this.filterableColumns !== null && name in this.filterableColumns) {
            return this.filterableColumns[name];
        }
        throw new Error(`[${name}] is not allowed for filtering`);
    }
    static __type = ['T', () => __ΩRecord, () => __ΩICallbackFilter, 'filterableColumns', function () { return null; }, () => __ΩRecord, () => __ΩICallbackSorter, 'sortableColumns', function () { return null; }, 'searchableColumns', function () { return null; }, 'knex', 'constructor', () => __ΩIGridRequestDto, 'dto', 'searchQuery', () => __ΩRecord, 'data', 'toObject', () => __ΩIGridRequestDto, () => GridResponse_1.default, 'filter', 'countQuery', 'query', () => __ΩIGridRequestDtoFilter, 'addOrExpression', () => __ΩIGridRequestDto, 'addPaging', () => __ΩIGridRequestDto, 'addSorters', () => __ΩIGridRequestDto, 'addFilters', 'name', () => __ΩICallbackFilter, 'getFilterableColumn', 'b!P&Pn#&Jo"#,J3$<>%P&Pn\'&Jo&#,J3(<>)P&F,J3*<>+P"2,;9"0-P"2,n.2/"00<=P&#o1#22e"!03<=Pn42/Pe#!75`06P"2,"`07<P"28n926"0:;sP"28n;2/"0<;P"28n=2/"0>;P"28n?2/"0@;P&2APnB&J0C;5'];
}
exports.default = AKnexGrid;
