"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.__ΩICallbackSorter = exports.__ΩICallbackFilter = void 0;
const { __ΩIGridRequestDto } = require('./GridRequestDto');
const { __ΩOperator } = require('./GridRequestDto');
const { __ΩIGridRequestDtoFilter } = require('./GridRequestDto');
const { __ΩDirection } = require('./GridRequestDto');
function __assignType(fn, args) {
    fn.__type = args;
    return fn;
}
const __ΩRecord = ['K', 'T', 'l\'e#"Rb!b"Pde"!N#!'];
const crypto_1 = require("crypto");
const typeorm_1 = require("typeorm");
const GridRequestDto_1 = require("./GridRequestDto");
const GridResponse_1 = __importDefault(require("./GridResponse"));
const __ΩICallbackFilter = ['queryBuilder', () => __ΩOperator, 'operator', 'values', '', 'P"2!n"2#&F2$$/%'];
exports.__ΩICallbackFilter = __ΩICallbackFilter;
const __ΩICallbackSorter = [() => typeorm_1.SelectQueryBuilder, 'queryBuilder', () => __ΩDirection, 'direction', '', 'PP"7!2"n#2$$/%'];
exports.__ΩICallbackSorter = __ΩICallbackSorter;
class ATypeORMGrid {
    dataSource;
    constructor(dataSource) {
        this.dataSource = dataSource;
    }
    async filter(dto) {
        const queryBuilder = this.getQueryBuilder(this.dataSource.getRepository(this.getEntity()));
        this.addFilters(queryBuilder, dto);
        this.addSorters(queryBuilder, dto);
        this.addPaging(queryBuilder, dto);
        const [items, count] = await queryBuilder.getManyAndCount();
        return new GridResponse_1.default(items, count, dto);
    }
    addOrExpression(queryBuilder, column, operator, values) {
        const hashOne = (0, crypto_1.randomBytes)(10).toString('hex');
        const hashTwo = (0, crypto_1.randomBytes)(10).toString('hex');
        const getValueOne = __assignType(() => values[0] ?? (() => {
            throw new Error(`Operator ${operator}: Missing 1st value!`);
        })(), ['', 'P&/!']);
        const getValueTwo = __assignType(() => values[1] ?? (() => {
            throw new Error(`Operator ${operator}: Missing 2nd value!`);
        })(), ['', 'P&/!']);
        switch (operator) {
            case GridRequestDto_1.Operator.EQ:
                queryBuilder.orWhere(`${column} = :${hashOne}`, { [hashOne]: getValueOne() });
                break;
            case GridRequestDto_1.Operator.NEQ:
                queryBuilder.orWhere(`${column} != :${hashOne}`, { [hashOne]: getValueOne() });
                break;
            case GridRequestDto_1.Operator.IN:
                queryBuilder.orWhere(`${column} IN (:${hashOne})`, { [hashOne]: values });
                break;
            case GridRequestDto_1.Operator.NIN:
                queryBuilder.orWhere(`${column} NOT IN (:${hashOne})`, { [hashOne]: values });
                break;
            case GridRequestDto_1.Operator.GT:
                queryBuilder.orWhere(`${column} > :${hashOne}`, { [hashOne]: getValueOne() });
                break;
            case GridRequestDto_1.Operator.LT:
                queryBuilder.orWhere(`${column} < :${hashOne}`, { [hashOne]: getValueOne() });
                break;
            case GridRequestDto_1.Operator.GTE:
                queryBuilder.orWhere(`${column} >= :${hashOne}`, { [hashOne]: getValueOne() });
                break;
            case GridRequestDto_1.Operator.LTE:
                queryBuilder.orWhere(`${column} <= :${hashOne}`, { [hashOne]: getValueOne() });
                break;
            case GridRequestDto_1.Operator.LIKE:
                queryBuilder.orWhere(`${column} LIKE :${hashOne}`, { [hashOne]: `%${getValueOne()}%` });
                break;
            case GridRequestDto_1.Operator.STARTS:
                queryBuilder.orWhere(`${column} LIKE :${hashOne}`, { [hashOne]: `${getValueOne()}%` });
                break;
            case GridRequestDto_1.Operator.ENDS:
                queryBuilder.orWhere(`${column} LIKE :${hashOne}`, { [hashOne]: `%${getValueOne()}` });
                break;
            case GridRequestDto_1.Operator.NEMPTY:
                queryBuilder.orWhere(`${column} IS NOT NULL`);
                break;
            case GridRequestDto_1.Operator.EMPTY:
                queryBuilder.orWhere(`${column} IS NULL`);
                break;
            case GridRequestDto_1.Operator.BETWEEN:
                queryBuilder.orWhere(`${column} BETWEEN :${hashOne} AND :${hashTwo}`, {
                    [hashOne]: getValueOne(),
                    [hashTwo]: getValueTwo(),
                });
                break;
            case GridRequestDto_1.Operator.NBETWEEN:
                queryBuilder.orWhere(`${column} NOT BETWEEN :${hashOne} AND :${hashTwo}`, {
                    [hashOne]: getValueOne(),
                    [hashTwo]: getValueTwo(),
                });
                break;
            default:
                queryBuilder.orWhere(`${column} = :${hashOne}`, { [hashOne]: getValueOne() });
                break;
        }
    }
    getFilterColumn(key) {
        const filters = this.getFilters();
        if (key in filters) {
            return filters[key];
        }
        throw new Error(`Column ${key} cannot be used for filtering!`);
    }
    getSorterColumn(key) {
        const sorters = this.getSorters();
        if (key in sorters) {
            return sorters[key];
        }
        throw new Error(`Column ${key} cannot be used for sorting!`);
    }
    addFilters(queryBuilder, dto) {
        const andFilters = [...dto.filter ?? [], ...dto.additionalFilter ?? []];
        for (const orFilters of andFilters) {
            queryBuilder.andWhere(new typeorm_1.Brackets(__assignType((innerQueryBuilder) => {
                for (const orFilter of orFilters) {
                    this.addFilter(innerQueryBuilder, orFilter);
                }
            }, ['innerQueryBuilder', '', 'P"2!"/"'])));
        }
        const { search } = dto;
        const searches = this.getSearches();
        if (search && searches.length) {
            queryBuilder.andWhere(new typeorm_1.Brackets(__assignType((innerQueryBuilder) => {
                for (const innerSearch of this.getSearches()) {
                    this.addFilter(innerQueryBuilder, {
                        column: innerSearch,
                        operator: GridRequestDto_1.Operator.LIKE,
                        value: [search],
                    });
                }
            }, ['innerQueryBuilder', '', 'P"2!"/"'])));
        }
    }
    addSorters(queryBuilder, dto) {
        const sorters = [...dto.sorter ?? [], ...dto.additionalSorter ?? []];
        for (const sorter of sorters) {
            const innerSorter = this.getSorterColumn(sorter.column);
            if (typeof innerSorter === 'string') {
                queryBuilder.addOrderBy(innerSorter, sorter.direction);
            }
            else {
                innerSorter(queryBuilder, sorter.direction);
            }
        }
    }
    addFilter(queryBuilder, { column, operator, value }) {
        const innerColumn = this.getFilterColumn(column);
        if (typeof innerColumn === 'string') {
            this.addOrExpression(queryBuilder, innerColumn, operator, value);
        }
        else {
            queryBuilder.orWhere(new typeorm_1.Brackets(__assignType((innerQueryBuilder) => {
                innerColumn(innerQueryBuilder, operator, value);
            }, ['innerQueryBuilder', '', 'P"2!"/"'])));
        }
    }
    addPaging(queryBuilder, dto) {
        const paging = dto.paging ?? {
            page: 1,
            itemsPerPage: 10,
        };
        paging.page = Math.max(paging.page, 1);
        paging.itemsPerPage = Math.max(paging.itemsPerPage, 1);
        dto.paging = paging;
        queryBuilder
            .take(paging.itemsPerPage)
            .skip((paging.page - 1) * paging.itemsPerPage);
    }
    static __type = ['T', () => typeorm_1.DataSource, 'dataSource', 'constructor', 'getEntity', () => typeorm_1.Repository, 'repository', () => typeorm_1.SelectQueryBuilder, 'getQueryBuilder', () => __ΩRecord, () => __ΩICallbackFilter, 'getFilters', () => __ΩRecord, () => __ΩICallbackSorter, 'getSorters', 'getSearches', () => __ΩIGridRequestDto, 'dto', () => GridResponse_1.default, 'filter', 'queryBuilder', 'column', () => __ΩOperator, 'operator', 'values', 'addOrExpression', 'key', () => __ΩICallbackFilter, 'getFilterColumn', () => __ΩICallbackSorter, 'getSorterColumn', () => typeorm_1.SelectQueryBuilder, () => __ΩIGridRequestDto, 'addFilters', () => typeorm_1.SelectQueryBuilder, () => __ΩIGridRequestDto, 'addSorters', () => __ΩIGridRequestDtoFilter, 'param1', 'addFilter', () => typeorm_1.SelectQueryBuilder, () => __ΩIGridRequestDto, 'addPaging', 'b!PP7"2#;9"0$P"0%<=PPe#!7&2\'Pe#!7(0)<=P&Pn+&Jo*#0,<=P&Pn.&Jo-#0/<=P&F00<=Pn122Pe#!73`04P"25&26n728&F29$0:<P&2;Pn<&J0=;P&2;Pn>&J0?;PPe#!7@25nA22$0B;PPe#!7C25nD22$0E;P"25nF2G$0H;PPe#!7I25nJ22$0K;5'];
}
exports.default = ATypeORMGrid;
